const express = require('express');
const bcrypt = require('bcrypt');
const User = require('../models/user');
const render = require('../utils/render');
const { warn, error } = require('../utils/notifications');

const router = express.Router();

router.get('/signup', (req, res) => {
  return render(req, res, 'signup');
});

router.post('/signup', (req, res) => {
  const passwordHash = bcrypt.hashSync(req.body.password, config.cryptRounds);
  User.create(req.body, (err, user) => {
    err ? error(req, res, 'Error creating user', err) : null;
    user.passwordHash = passwordHash;
    user.save();
    req.session.user = user;
    return res.redirect('/');
  });
});

router.post('/login', (req, res) => {
  if (!req.body.username || !req.body.password) {
    return res.redirect('/auth/signup');
  } else if (req.session.user) {
    error(req, res, 'User already logged in', 'You must logout before log in.')
  } else {
    User.findOne({ username: req.body.username }, (err, user) => {
      err ? error(req, res, 'Error fetching user', err) : null;
      if (bcrypt.compareSync(req.body.password, user.passwordHash)) {
        req.session.user = user;
        return res.redirect(req.query.nextUrl || '/');
      } else {
        error(req, res, 'Bad credentials')
        return res.redirect('/auth/signup');
      }
    });
  }
});

router.post('/logout', (req, res) => {
  req.session.destroy();
  return res.redirect('/');
});

module.exports = router;