const express = require('express');
const fs = require('fs');
const { initAttributesPoints, costTable, attributes, maxActionPoints, maxCharactersNumber } = require('js-yaml').safeLoad(fs.readFileSync('./config.yml', 'utf8'));
const Character = require('../models/character');
const render = require('../utils/render');
const { warn, error } = require('../utils/notifications');

const router = express.Router();

const getScore = (data) => Object.keys(data).reduce((total, el) => total + costTable[data[el]], 0);

router.get('/', async (req, res) => {
  // TODO: Login required
  const avatars = fs.readdirSync('media/avatars');
  Character.find({ user: req.session.user._id }, (err, characters) => {
    return render(req, res, 'characters', { characters, avatars, initAttributesPoints, maxActionPoints });
  });
});

// router.post('/generate', async (req, res) => {
//   // TODO: Login required
//   if (!req.body.name) {
//     return error(req, res, 'Name required');
//   }
//   const character = await Character.generateCharacter(req.body.name, req.session.user._id, req.body.avatar);
//   return res.redirect('/characters');
// });

router.post('/create', async (req, res) => {
  const validatedAttributes = attributes.reduce((obj, name) => {
      obj[name] = Number(req.body[name]);
      return obj;
    }, {});
  if (getScore(validatedAttributes) > initAttributesPoints || attributes.map(name => validatedAttributes[name]).some(value => value > 18 || value < 7)) {
    return error(req, res, 'Bad form', "Character creation corrupted");
  } else if (maxCharactersNumber <= await Character.countDocuments({ user: req.session.user._id })) {
    return error(req, res, 'Too many characters', "You are not allowed to create one more character.");
  }
  await Character.create({
    user: req.session.user._id,
    name: req.body.name,
    avatar: req.body.avatar,
    attributes: validatedAttributes
  });
  res.redirect('/characters');
});

module.exports = router;